/*
 * @eventCSVFile.java Version 1.1 02/18/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package event.io;

import java.awt.*;
import java.io.*;
import java.lang.*;
import java.util.*;

import util.utilFileIO;
import cmn.cmnString;
import rock.rockDataStruct;

import event.eventFileDataStruct;     // Zonation File Data Structure
import iqstrat.iqstratHeadersStruct;  // IQSTRAT Headers Data Structure

/** Class eventCSVFile
 *  <p> This Class is designed to open an ascii text rock data CSV file and read
 *      the contents and parse the data into the rock Structure.
 *
 *  @version 1.1 02/18/2015
 *  @author  John R. Victorine
 */

public class eventCSVFile
{
  public static final int _TOP  = event.eventStandardTools._TOP;
  public static final int _BASE = event.eventStandardTools._BASE;

  public static final int _NAME = event.eventStandardTools._NAME;

  public static final int _GR   = event.eventStandardTools._GR;
  public static final int _CGR  = event.eventStandardTools._CGR;

  public static final int _PE   = event.eventStandardTools._PE;
  public static final int _RHOB = event.eventStandardTools._RHOB;
  public static final int _DPHI = event.eventStandardTools._DPHI;
  public static final int _NPHI = event.eventStandardTools._NPHI;

  public static final int _SPHI = event.eventStandardTools._SPHI;
  public static final int _DTC  = event.eventStandardTools._DTC;
  public static final int _DTS  = event.eventStandardTools._DTS;
  public static final int _DTSF = event.eventStandardTools._DTSF;
  public static final int _DTSS = event.eventStandardTools._DTSS;

  public static final int _TH   = event.eventStandardTools._TH;
  public static final int _U    = event.eventStandardTools._U;
  public static final int _K    = event.eventStandardTools._K;

  public static final int TOTAL = event.eventStandardTools.TOTAL_LAS;

  private utilFileIO           pFILE    = null;

  private String               sLine1   = "";
  private String               sLine2   = "";
  private eventFileDataStruct  stData   = null;
  private iqstratHeadersStruct stHead   = null;

  private int                  iRows    = 0;
  private int                  iColumns = 0;
  private int                  iMAXIMUM = 0;

  private int                  iError   = util.utilFileIO.NOERROR;
  private String               sError   = new String("");

  /** Construct eventCSVFile()
   *  <p> This is the Constructor for this class.
   */

  public eventCSVFile() { }

  /* ====================================================================== *
   * ---------------------------- FILE METHODS ---------------------------- *
   * ====================================================================== */

  /** Method Open()
    * <p> This is the Constructor for this class.
    * @param iFileID    = To access the file as URL or FILE.
    * @param sDirectory = File URL or Directory Path
    * @param sFilename  = File Name
    */

  public void Open(int iFileID, String sDirectory, String sFilename)
  {
    pFILE    = new utilFileIO();
    pFILE.Open(iFileID, util.utilFileIO.READ, sDirectory, sFilename);
    iError   = pFILE.getErrorID();
    sError   = pFILE.getError();
    iRows    = 0;
    iColumns = 0;
  }

  /** Method Close()
   *  <p> This method closes the File.
   */

  public void Close()
  {
	iMAXIMUM = iRows;
	pFILE.Close();
  }

  /** Method delete()
   * <p> This method will set all the structures to null to force java to free
   *     the allocated memory
   */

  public void delete()
  {
    sLine1 = null;
    sLine2 = null;
    sError = null;

    if (pFILE != null)
      pFILE.delete();
    pFILE = null;
  }

  /** Method get2Rows()
   * <p> This method will read and parse the File and get the first 2 lines.
   */

  public void get2Rows()
  {
    int             iCount  = 0;
    String          sTemp   = "";
    StringTokenizer stToken = null;

    if (iError == util.utilFileIO.NOERROR)
    {
      while (!pFILE.Next().equals(util.utilFileIO.EOF))
      {
        iError = pFILE.getErrorID();
        sError = pFILE.getError();

        sTemp = pFILE.getString();
        if (iCount == 0)
        {
          sLine1   = new String(sTemp);
          stToken  = new StringTokenizer(sTemp, ",");
          iColumns = stToken.countTokens();
        }

        if (iCount == 1)
          sLine2 = new String(sTemp);

        iCount++;
        iRows++;
      }
    }
  }

  /** Method getHeader()
   *  <p> This method will retrieve the Header information from the first
   *      line of Zonation file.
   */

  public iqstratHeadersStruct getHeader()
  {
	iqstratHeadersStruct st        = new iqstratHeadersStruct();
    String               sTemp     = "";
    StringTokenizer      stToken   = null;
    String               sTokens[] = null;
    String               sData[]   = null;
    int                  iCount    = 0;

	if (sLine1.length() > 0)
	{
      sTemp   = new String(sLine1.replace('\t', ' '));
      sTokens = las3.las3Parse.splitDataLine( sTemp, ";" );

      iCount = 0;
      if (sTokens != null)
        iCount = sTokens.length;

      for (int i=0; i<iCount; i++)
      {
		sTemp = new String(sTokens[i].trim());
		sData = las3.las3Parse.splitDataLine( sTemp, "=" );

		if (sData.length == 2)
		{
		  if (sData[0].equals("WELL"))
		  {
			st.sName = new String(sData[1]);
		  }

		  if (sData[0].equals("API"))
		  {
			st.sAPI = new String(sData[1]);
		  }

		  if (sData[0].equals("LAT"))
		  {
            if (cmn.cmnString.isNumeric(sData[1]))
            {
			  st.dLatitude = cmn.cmnString.stringToDouble(sData[1]);
		    }
		  }

		  if (sData[0].equals("TRS"))
		  {
			st = las3.las3LoadWellHeader.parseSTR( sData[1], st );
		  }

		  if (sData[0].equals("LNG"))
		  {
            if (cmn.cmnString.isNumeric(sData[1]))
            {
			  st.dLongitude = cmn.cmnString.stringToDouble(sData[1]);
		    }
		  }

		  if (sData[0].equals("GL"))
		  {
            if (cmn.cmnString.isNumeric(sData[1]))
            {
			  st.dGL = cmn.cmnString.stringToDouble(sData[1]);
		    }
		  }

		  if (sData[0].equals("KB"))
		  {
            if (cmn.cmnString.isNumeric(sData[1]))
            {
			  st.dKB = cmn.cmnString.stringToDouble(sData[1]);
		    }
		  }

		  if (sData[0].equals("DF"))
		  {
            if (cmn.cmnString.isNumeric(sData[1]))
            {
			  st.dDF = cmn.cmnString.stringToDouble(sData[1]);
		    }
		  }
		}
	  }
	}

	return (st);
  }

  /** Method getData()
   * <p> This method will read and parse the Rock File into its data structures.
   * @param iStart  = the starting row number
   * @param iColumn = Array of file column mapping to the rock Data structure
   */

  public void getData(int iStart, int iColumn[])
  {
    int                 i=0;
    int                 iCount = 1;
    String              sTemp  = "";
//    eventFileDataStruct st     = null;
    int                 iValue = 0;
    int                 iBegin = 0;
    int                 iRow   = 0;
    int                 iTotal = iMAXIMUM-iStart+1;

    stData = event.eventFileDataUtility.initData(iTotal);

    if (iError == util.utilFileIO.NOERROR)
    {
      while (!pFILE.Next().equals(util.utilFileIO.EOF))
      {
        iError = pFILE.getErrorID();
        sError = pFILE.getError();

        if (iError == util.utilFileIO.NOERROR)
        {
          if ((iCount >= iStart) && (iRow < iTotal))
          {
            sTemp  = pFILE.getString();
            stData = parseRow(sTemp, iRow, iColumn, stData);
            if (iRow > 0)
            {
			  if (stData.dBase[iRow-1] < stData.dTop[iRow])
			    stData.dBase[iRow-1] = stData.dTop[iRow];
			}
            iRow++;
            stData.iRows = iRow;
          }

          iCount++;
        }
      }
    }
  }

  /** Method parseRow()
   * <p> This method will parse a Rock File row into the rock structure.
   * @param  sData   = Data Row String
   * @param  iRow    = The Row Number
   * @param  iColumn = The mapped columns
   * @return st      = the LAS Data Structure
   */

  private eventFileDataStruct parseRow( String sData, int iRow, int iColumn[],
                                        eventFileDataStruct st)
  {
    int                 iFound    = 0;
    int                 iNext     = 0;
    String              sTemp     = "";
    StringTokenizer     stToken   = null;
    String              sTokens[] = null;

    sTemp    = new String(sData.replace('\t', ' '));
    sTokens  = las3.las3Parse.splitDataLine( sTemp, "," );

    iColumns = 0;
    if (sTokens != null)
      iColumns = sTokens.length;

    for (int i=0; i<iColumns; i++)
    {
      sTemp  = sTokens[i];
      sTemp  = new String(sTemp.trim());

      iNext  = i+1;
      iFound = -1;
      for (int j=0; j<TOTAL; j++)
      {
        if (iNext == iColumn[j])
          iFound = j;
      }

      if (iFound > -1)
        st = mapData(iRow, sTemp, event.eventStandardTools.CURVES[iFound][1], st);
    }

    return (st);
  }

  /** Method mapData()
   * <p> This method will parse a Rock File row into the rock structure.
   * @param  iRow        = Row Number
   * @param  sData       = Data Row String
   * @param  sIdentifier = the column idenfier
   * @param  st          = the LAS Data Structure
   * @return st          = the LAS Data Structure
   */

  private eventFileDataStruct mapData( int    iRow,
	                                   String sData,
                                       String sIdentifier,
                                       eventFileDataStruct st )
  {
    String          sTemp     = "";
    StringTokenizer stToken   = null;
    String          sTokens[] = null;
    int             len       = 0;

    // Name of bed

    if (sIdentifier.equals(event.eventStandardTools.CURVES[_NAME][1]))
    {
      sTemp    = new String(sData.replace('\t', ' '));
      sTokens  = las3.las3Parse.splitDataLine( sTemp, " " );

      len = 0;
      if (sTokens != null)
        len = sTokens.length;

      if (len >= 1)
      {
		if (!(sTokens[0].toLowerCase()).equals("group"))
		{
		  st.sName[iRow] = new String(sData);
		}
	  }
	}

    // Depth Top of LAS Curve

    if (sIdentifier.equals(event.eventStandardTools.CURVES[_TOP][1]))
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        stData.iTOP   = 1;
        st.dTop[iRow] = cmn.cmnString.stringToDouble(sData);
      }
    }

    // Depth Base of LAS Curve

    if (sIdentifier.equals(event.eventStandardTools.CURVES[_BASE][1]))
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        stData.iBASE   = 1;
        st.dBase[iRow] = cmn.cmnString.stringToDouble(sData);
      }
    }

    if (sIdentifier.equals(event.eventStandardTools.CURVES[_GR][1]))
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        stData.iGR   = 1;
        st.dGR[iRow] = cmn.cmnString.stringToDouble(sData);
      }
    }

    if (sIdentifier.equals(event.eventStandardTools.CURVES[_CGR][1]))
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        stData.iCGR   = 1;
        st.dCGR[iRow] = cmn.cmnString.stringToDouble(sData);
      }
    }

    if (sIdentifier.equals(event.eventStandardTools.CURVES[_PE][1]))
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        stData.iPE   = 1;
        st.dPE[iRow] = cmn.cmnString.stringToDouble(sData);
      }
    }

    if (sIdentifier.equals(event.eventStandardTools.CURVES[_RHOB][1]))
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        stData.iRHOB   = 1;
        st.dRHOB[iRow] = cmn.cmnString.stringToDouble(sData);
      }
    }

    if (sIdentifier.equals(event.eventStandardTools.CURVES[_DPHI][1]))
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        stData.iDPHI   = 1;
        st.dDPHI[iRow] = cmn.cmnString.stringToDouble(sData);
      }
    }

    if (sIdentifier.equals(event.eventStandardTools.CURVES[_NPHI][1]))
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        stData.iNPHI   = 1;
        st.dNPHI[iRow] = cmn.cmnString.stringToDouble(sData);
      }
    }

    if (sIdentifier.equals(event.eventStandardTools.CURVES[_SPHI][1]))
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        stData.iSPHI   = 1;
        st.dSPHI[iRow] = cmn.cmnString.stringToDouble(sData);
      }
    }

    if (sIdentifier.equals(event.eventStandardTools.CURVES[_DTC][1]))
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        stData.iDTC   = 1;
        st.dDTC[iRow] = cmn.cmnString.stringToDouble(sData);
      }
    }

    if (sIdentifier.equals(event.eventStandardTools.CURVES[_DTS][1]))
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        stData.iDTS   = 1;
        st.dDTS[iRow] = cmn.cmnString.stringToDouble(sData);
      }
    }
    if (sIdentifier.equals(event.eventStandardTools.CURVES[_DTSF][1]))
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        stData.iDTSF   = 1;
        st.dDTSF[iRow] = cmn.cmnString.stringToDouble(sData);
      }
    }
    if (sIdentifier.equals(event.eventStandardTools.CURVES[_DTSS][1]))
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        stData.iDTSS   = 1;
        st.dDTSS[iRow] = cmn.cmnString.stringToDouble(sData);
      }
    }

    if (sIdentifier.equals(event.eventStandardTools.CURVES[_TH][1]))
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        stData.iTh   = 1;
        st.dTh[iRow] = cmn.cmnString.stringToDouble(sData);
      }
    }

    if (sIdentifier.equals(event.eventStandardTools.CURVES[_U][1]))
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        stData.iU   = 1;
        st.dU[iRow] = cmn.cmnString.stringToDouble(sData);
      }
    }

    if (sIdentifier.equals(event.eventStandardTools.CURVES[_K][1]))
    {
      if (cmn.cmnString.isNumeric(sData))
      {
        stData.iK   = 1;
        st.dK[iRow] = cmn.cmnString.stringToDouble(sData);
      }
    }

    return (st);
  }

  /* ====================================================================== *
   * ----------------------------- GET METHODS ---------------------------- *
   * ====================================================================== */

  /** Method getLine1()
   * <p> This method will return the 1st Line of the delimited file
   * @return sLine1 = 1st line of delimited file
   */

  public String getLine1() { return (sLine1); }

  /** Method getLine2()
   * <p> This method will return the 2nd Line of the delimited file
   * @return sLine2 = 2nd line of delimited file
   */

  public String getLine2() { return (sLine2); }

  /** Method getData()
   * <p> This method will return rock data list
   * @return stData = rock Data list structure
   */

  public eventFileDataStruct getData() { return (stData); }

  /* ====================================================================== *
   * -------------------------- GET ERROR METHODS ------------------------- *
   * ====================================================================== */

  /** METHOD getErrorID()
   *  <p> This method will return the error number.
   *  @return iError = Error Indicator
   */

  public int getErrorID() { return (iError); }

  /** METHOD getError()
   *  <p> This method will return the error string.
   *  @return sError = Error String
   */

  public String getError() { return (sError); }
}

/*
 *  @version 1.1 02/18/2015
 *  @author  John Victorine
 */
