/*
 * @quakeSensorDataTable.java Version 1.1 05/21/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake.gui;

import java.awt.*;
import java.lang.*;
import javax.swing.*;
import javax.swing.table.*;

import gui.guiTable;

import quake.quakeSensorsDataListStruct;
import quake.quakeSensorsDataStruct;

/** Class quakeSensorDataTable
 *  <p> This Class will display the Computed Seismic Event Data.
 *
 *  @version 1.1 05/21/2015
 *  @author  John R. Victorine
 */

public class quakeSensorDataTable
{
  private int      iRows     = 0;
  private int      iColumns  = 9;
  private String   sColumn[] = {"Date Time",
                                "Channel",
                                "1",
                                "2",
                                "3",
                                "Magnitude",
                                "Latitude",
                                "Longitude",
                                "Depth" };
  private int     iColLength[] = {30,  // Date Time
                                   8,  // Channel
                                   4,  // 1
                                   4,  // 2
                                   4,  // 3
                                  12,  // Magnitude
                                  12,  // Latitude
                                  12,  // Longitude
                                  12}; // Depth
  private Object   oData[][] = null;
  private guiTable pTable    = null;

  private quakeSensorsDataListStruct st      = null;
  private int                        iChange = 0;

  /** Constructor quakeSensorDataTable()
   *  <p> This is the Constructor for this class.
   */

  public quakeSensorDataTable()
  {
    pTable = new guiTable(0, iColumns, sColumn);
    resetColumnWidths();
    pTable.showHorScroll(true);
  }

  /** Constructor quakeSensorDataTable()
   *  <p> This is the Constructor for this class.
   * @param st     = The Headers List Data Structure
   */

  public quakeSensorDataTable(quakeSensorsDataListStruct st)
  {
    this.st = st;

    populateList();

    if (iRows > 0)
      pTable = new guiTable(0, iRows, iColumns, sColumn, oData);
    else
      pTable = new guiTable(0, iColumns, sColumn);

    resetColumnWidths();
    pTable.showHorScroll(true);

    if (iChange == 0)
      iRows = 0;

    if (iRows > 0)
      pTable.setSelectedRow(0);
  }

  /** Method resetColumnWidths()
   *  <p> This method will reset the column widths to a specific size.
   */

  private void resetColumnWidths()
  {
    for (int i=0; i<iColumns; i++)
    {
      pTable.resizeColumn(i, iColLength[i]);
    }
  }

  /** Method close()
   * <p> This method will set global variables to null in order to force
   *     Java to deallocate memory
   */

  public void close()
  {
    sColumn    = null;
    iColLength = null;
    oData      = null;
    pTable     = null;

    st         = null;
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method populateList()
   *  <p> This method will populate the Region List data.
   */

  private void populateList()
  {
    int i,j;
    int iCount = 0;
    int iSensor[] = { 0, 0, 0 };

    iChange = 0;
    iRows   = 1;

    oData = new String[1][iColumns];
    for (i=0; i<iColumns; i++)
    {
      oData[0][i] = new String("");
    }

    if (st != null)
    {
      if (st.iCount > 0)
      {
        iChange = 1;
        iCount  = 0;
        oData   = new Object[st.iCount][iColumns];

        for (i=0; i<st.iCount; i++)
        {
		  for (j=0; j<3; j++)
		    iSensor[j] = (int) st.stItem[i].data[j][0];

          oData[iCount][0] = new String(st.stItem[i].sTime);
          oData[iCount][1] = new String(st.stItem[i].sChannel);
          oData[iCount][2] = new String(""+iSensor[0]);
          oData[iCount][3] = new String(""+iSensor[1]);
          oData[iCount][4] = new String(""+iSensor[2]);
          oData[iCount][5] = new Double(st.stItem[i].dMagnitude);
          oData[iCount][6] = new Double(st.stItem[i].dLatitude);
          oData[iCount][7] = new Double(st.stItem[i].dLongitude);
          oData[iCount][8] = new Double(st.stItem[i].depth);

          iCount++;
        }

        iRows = iCount;
      }
    }
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setData()
   *  <p> This method will reload the Region List after adding, deleting from
   *      the formation name list.
   * @param st     = The Regions List Data Structure
   */

  public void setData(quakeSensorsDataListStruct st)
  {
    this.st = st;

    populateList();

    pTable.reload(iRows, oData);

    if (iChange == 0)
      iRows = 0;

    if (iRows > 0)
      pTable.setSelectedRow(0);
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getTotalRows()
   *  <p> This method will return the total rows in the table.
   *  @return Total Number of Rows.
   */

  public int getTotalRows() { return (iRows); }

  /** Method getRow()
   *  <p> This method will return the selected data id in the table.
   *  @return selected data id.
   */

  public int getRow() { return (pTable.getSelectedRow()); }

  /** Method getRowData()
   *  <p> This method will return the Region Data Structure of the
   *      selected row.
   *  @return stData = Region Data Structure.
   */

  public quakeSensorsDataStruct getRowData()
  {
    quakeSensorsDataStruct stData = null;

    stData = st.stItem[pTable.getSelectedRow()];

    return (stData);
  }

  /** Method getAllData()
   *  <p> This method will returns the Region List Data Structure.
   *  @return st = Region List Data Structure.
   */

  public quakeSensorsDataListStruct getAllData() { return (st); }

  /** Method getScrollPane()
   *  <p> This method will return the scroll pane.
   *  @return jScrollPane - Scroll Pane containing the Table.
   */

  public JScrollPane getScrollPane() { return (pTable.getScrollPane()); }
}

/*
 *  @version 1.1 05/21/2015
 *  @author  John Victorine
 */
