/*
 * @quakeSensorsDataUtility.java Version 1.1 05/21/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake;

import quake.quakeSensorsDataListStruct;
import quake.quakeSensorsDataStruct;

/** CLASS quakeSensorsDataUtility
 *  <p> This file creates utility methods for quake sensor data structures.
 *
 *  @version 1.1 05/21/2015
 *  @author  John R. Victorine
 */

public class quakeSensorsDataUtility
{
  /** Method copyList()
   * <p> This method will copy one structure to another
   * @param  stOld = the Old List Structure
   * @return stNew = the New List structure
   */

  public static quakeSensorsDataListStruct copyList(quakeSensorsDataListStruct stOld)
  {
    quakeSensorsDataListStruct stNew = null;

    if (stOld != null)
    {
      stNew        = new quakeSensorsDataListStruct();
      stNew.iCount = stOld.iCount;
      stNew.stItem = new quakeSensorsDataStruct[stOld.iCount];

      for (int i=0; i<stOld.iCount; i++)
      {
        stNew.stItem[i] = copy(stOld.stItem[i]);
      }
    }

    return (stNew);
  }

  /** Method transfer()
   * <p> This method will transfer one structure to another
   * @param  stOld = the Old List Structure
   * @return stNew = the New List structure
   */

  public static quakeSensorsDataListStruct transfer(quakeSensorsDataListStruct stOld)
  {
    quakeSensorsDataListStruct stNew = null;

    if (stOld != null)
    {
	  stNew = copyList( stOld );

	  stOld.delete();
	  stOld = null;
	}

    return (stNew);
  }

  /** Method copy()
   * <p> This method will copy data from one structure to another
   * @param  stOld = The original Data Structure
   * @return stNew = The new Data Structure
   */

  public static quakeSensorsDataStruct copy(quakeSensorsDataStruct stOld)
  {
    quakeSensorsDataStruct stNew = new quakeSensorsDataStruct();

    if (stOld != null)
    {
      stNew.sKEY       = new String(stOld.sKEY);

      // Time and Computed location of seismic event.

      stNew.sTime      = new String(stOld.sTime);
      stNew.sChannel   = new String(stOld.sChannel);
      stNew.dMagnitude = stOld.dMagnitude;
      stNew.dLatitude  = stOld.dLatitude;
      stNew.dLongitude = stOld.dLongitude;
      stNew.depth      = stOld.depth; // Elevation [ft]

      // Sensor Raw data for computation of seismic event.

      for (int i=0; i<3; i++)
      {
		for (int j=0; j<5; j++)
		{
          stNew.data[i][j] = stOld.data[i][j];
		}
	  }
    }

    return (stNew);
  }

  /** Method add()
   * <p> This method will add the to an existing list
   * @param  stBlock = The Block List Data Structure
   * @param  st      = The Old List Data Structure
   * @return st      = The new List.
   */

  public static quakeSensorsDataListStruct add(
	  quakeSensorsDataStruct stBlock,
      quakeSensorsDataListStruct st)
  {
    int    i=0;
    int    iRecords = 0;
    int    iCount   = 0;
    quakeSensorsDataListStruct stTemp = null;

    if (st != null)
      iRecords = st.iCount+1;
    else
      iRecords = 1;

    stTemp        = new quakeSensorsDataListStruct();
    stTemp.stItem = new quakeSensorsDataStruct[iRecords];

    if (st != null)
    {
      if (st.iCount > 0)
      {
        for (i=0; i<st.iCount; i++)
        {
          if (iCount < iRecords)
          {
            stTemp.stItem[iCount] = copy(st.stItem[i]);
            iCount++;
          }
        }
      }

      st.delete();
    }

    stTemp.stItem[iCount] = new quakeSensorsDataStruct();
    stTemp.stItem[iCount] = copy(stBlock);
    iCount++;

    stTemp.iCount = iCount;

    st        = new quakeSensorsDataListStruct();
    st.stItem = new quakeSensorsDataStruct[iCount];
    st.iCount = iCount;

    for (i=0; i<stTemp.iCount; i++)
    {
      st.stItem[i] = copy(stTemp.stItem[i]);
    }

    stTemp.delete();

    return (st);
  }

  /** Method modify()
   * <p> This method will modify a data structure in an existing list
   * @param  sKEY    = The Primary Key of the data to be removed
   * @param  stBlock = The structure that will be replaced
   * @param  st      = The Old List Data Structure
   * @return st      = The new List.
   */

  public static quakeSensorsDataListStruct modify(
	  String sKEY,
      quakeSensorsDataStruct stBlock,
      quakeSensorsDataListStruct st)
  {
    int i=0;
    quakeSensorsDataListStruct stTemp = null;

    if (st != null)
    {
      stTemp        = new quakeSensorsDataListStruct();
      stTemp.stItem = new quakeSensorsDataStruct[st.iCount];
      stTemp.iCount = st.iCount;

      if (st.iCount > 0)
      {
        for (i=0; i<st.iCount; i++)
        {
          if (sKEY.equals(st.stItem[i].sKEY))
          {
            stTemp.stItem[i] = copy(stBlock);
          }
          else
          {
            stTemp.stItem[i] = copy(st.stItem[i]);
          }
        }
      }

      st.delete();

      st        = new quakeSensorsDataListStruct();
      st.stItem = new quakeSensorsDataStruct[stTemp.iCount];
      st.iCount = stTemp.iCount;

      for (i=0; i<stTemp.iCount; i++)
      {
        st.stItem[i] = copy(stTemp.stItem[i]);
      }

      stTemp.delete();
    }

    return (st);
  }

  /** Method remove()
   * <p> This method will remove a data structure from an existing list
   * @param  sKEY   = The Primary Key of the data to be removed
   * @param  st     = The Old List Data Structure
   * @return st     = The new List.
   */

  public static quakeSensorsDataListStruct remove(
	  String sKEY,
	  quakeSensorsDataListStruct st )
  {
    int i=0;
    int    iRecords = 0;
    int    iCount   = 0;
    int    iFound   = -1;
    quakeSensorsDataListStruct stTemp = null;

    if (st.iCount == 1)
    {
      st.delete();
      st = null;
    }
    else
    {
      if (st != null)
      {
        iRecords = st.iCount-1;

        stTemp        = new quakeSensorsDataListStruct();
        stTemp.stItem = new quakeSensorsDataStruct[iRecords];

        for (i=0; i<st.iCount; i++)
        {
          if (sKEY.equals(st.stItem[i].sKEY))
          {
            iFound = i;
          }
        }
      }

      if (st != null)
      {
        if (st.iCount > 0)
        {
          for (i=0; i<st.iCount; i++)
          {
            if (iCount < iRecords)
            {
              if (i != iFound)
              {
                stTemp.stItem[iCount] = copy(st.stItem[i]);
                iCount++;
              }
            }
          }
        }

        st.delete();
      }

      stTemp.iCount = iCount;

      st        = new quakeSensorsDataListStruct();
      st.stItem = new quakeSensorsDataStruct[iCount];
      st.iCount = iCount;

      for (i=0; i<stTemp.iCount; i++)
      {
        st.stItem[i] = copy(stTemp.stItem[i]);
      }

      stTemp.delete();
    }

    return (st);
  }
}

/*
 *  @version 1.1 05/21/2015
 *  @author  John Victorine
 */
